package consume

import (
	"applet/app/db"
	utils2 "applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	md2 "applet/es/md"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"github.com/streadway/amqp"
	"time"
)

func EggCanalUserVirtualCoinFlowAggregationConsume(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>EggCanalUserVirtualCoinFlowAggregationConsume>>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(50)
	delivery := ch.Consume(queue.Name, true) //设置自动应答

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			fmt.Println(">>>>>>>>>>>>>>>>>>EggCanalUserVirtualCoinFlowAggregationConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = handleEggCanalUserVirtualCoinFlowAggregationConsume(res.Body)
			if err != nil {
				fmt.Println("EggCanalUserVirtualCoinFlowAggregationConsume_ERR:::::", err.Error())
				utils2.FilePutContents("EggCanalUserVirtualCoinFlowAggregationConsume_ERR", utils2.SerializeStr(map[string]interface{}{
					"body": res.Body,
					"err":  err.Error(),
				}))
			}
			//_ = res.Reject(false)
			//_ = res.Ack(true)
		} else {
			panic(errors.New("error getting message"))
		}
	}
}

func handleEggCanalUserVirtualCoinFlowAggregationConsume(msg []byte) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.CanalUserVirtualCoinFlowAggregationMessage[md.CanalUserVirtualCoinFlowAggregation]
	err := json.Unmarshal(msg, &canalMsg)
	if err != nil {
		return nil
	}

	// 2. 监听插入信息
	if canalMsg.Type == md2.CanalMsgInsertSqlType && canalMsg.Data[0].Direction == utils2.IntToStr(1) {
		//2、查找基础设置
		settingDb := implement.NewEggEnergyBasicSettingDb(db.Db)
		basicSetting, err := settingDb.EggEnergyBasicSettingGetOne()
		if err != nil {
			return err
		}
		aggregationDb := implement.NewUserVirtualCoinFlowAggregationDb(db.Db)
		now := time.Now().AddDate(0, 0, 1)

		if canalMsg.Data[0].CoinId == utils2.IntToStr(basicSetting.PersonEggEnergyCoinId) {
			userVirtualCoinFlowAggregation, err1 := aggregationDb.UserVirtualCoinFlowAggregationGetOneByParams(map[string]interface{}{
				"key":   "uid",
				"value": canalMsg.Data[0].Uid,
			})
			if err1 != nil {
				return err1
			}
			if userVirtualCoinFlowAggregation == nil {
				//新增记录
				_, err3 := aggregationDb.UserVirtualCoinFlowAggregationInsert(&model.UserVirtualCoinFlowAggregation{
					Uid:           utils2.StrToInt(canalMsg.Data[0].Uid),
					CoinId:        utils2.StrToInt(canalMsg.Data[0].CoinId),
					TodayData:     canalMsg.Data[0].Amount,
					ThisWeekData:  canalMsg.Data[0].Amount,
					ThisMonthData: canalMsg.Data[0].Amount,
					NowData:       canalMsg.Data[0].Amount,
					CreateAt:      now.Format("2006-01-02 15:04:05"),
					UpdateAt:      now.Format("2006-01-02 15:04:05"),
				})
				if err3 != nil {
					return err3
				}
			} else {
				//更新记录
				amount, _ := decimal.NewFromString(canalMsg.Data[0].Amount)
				todayData, _ := decimal.NewFromString(userVirtualCoinFlowAggregation.TodayData)
				thisWeekData, _ := decimal.NewFromString(userVirtualCoinFlowAggregation.ThisWeekData)
				thisMonthData, _ := decimal.NewFromString(userVirtualCoinFlowAggregation.ThisMonthData)
				userVirtualCoinFlowAggregation.TodayData = todayData.Add(amount).String()
				userVirtualCoinFlowAggregation.ThisWeekData = thisWeekData.Add(amount).String()
				userVirtualCoinFlowAggregation.ThisMonthData = thisMonthData.Add(amount).String()
				userVirtualCoinFlowAggregation.NowData = canalMsg.Data[0].AfterAmount
				_, err2 := aggregationDb.UserVirtualCoinFlowAggregationUpdate(userVirtualCoinFlowAggregation.Id, userVirtualCoinFlowAggregation, "today_data", "this_week_data", "this_month_data", "now_data")
				if err2 != nil {
					return err2
				}
			}

			// 更新 user_egg_energy_aggregation
			uid := utils2.StrToInt64(canalMsg.Data[0].Uid)
			year, week := now.ISOWeek()
			_, month, day := now.Date()
			energyAggregationDb := implement.NewUserEggEnergyAggregationDb(db.Db)
			energyAggregation, err3 := energyAggregationDb.UserEggEnergyAggregationGetOneByTime(day, week, int(month), year, uid)
			if err3 != nil {
				return err3
			}
			amount, _ := decimal.NewFromString(canalMsg.Data[0].Amount)
			if energyAggregation == nil {
				// 不存在当天数据 创建新数据
				nowStr := now.Format("2006-01-02 15:04:05")
				m := model.UserEggEnergyAggregation{
					Uid:       uid,
					TodayData: canalMsg.Data[0].Amount,
					Day:       day,
					Week:      week,
					Month:     int(month),
					Year:      year,
					CreateAt:  nowStr,
					UpdateAt:  nowStr,
				}

				// 读取最新数据
				lastEnergyAggregation, err4 := energyAggregationDb.UserEggEnergyAggregationGetLastOneByUid(uid)
				if err4 != nil {
					return err4
				}
				// 判断是否上条数据是否在本周
				if lastEnergyAggregation != nil {
					if lastEnergyAggregation.Week == week && lastEnergyAggregation.Year == year {
						// 周数相同 累加上周数据
						thisWeekData, _ := decimal.NewFromString(lastEnergyAggregation.ThisWeekData)
						m.ThisWeekData = thisWeekData.Add(amount).String()
					} else {
						m.ThisWeekData = amount.String()
					}
					if lastEnergyAggregation.Month == int(month) && lastEnergyAggregation.Year == year {
						// 月份相同 累加上月数据
						thisMonthData, _ := decimal.NewFromString(lastEnergyAggregation.ThisMonthData)
						m.ThisMonthData = thisMonthData.Add(amount).String()
					} else {
						m.ThisMonthData = amount.String()
					}
				} else {
					m.ThisWeekData = amount.String()
					m.ThisMonthData = amount.String()
				}

				_, err5 := energyAggregationDb.UserEggEnergyAggregationInsert(&m)
				if err5 != nil {
					return err5
				}
			} else {
				// 存在当天数据累加数据
				todayData, _ := decimal.NewFromString(energyAggregation.TodayData)
				thisWeekData, _ := decimal.NewFromString(energyAggregation.ThisWeekData)
				thisMonthData, _ := decimal.NewFromString(energyAggregation.ThisMonthData)
				energyAggregation.TodayData = todayData.Add(amount).String()
				energyAggregation.ThisWeekData = thisWeekData.Add(amount).String()
				energyAggregation.ThisMonthData = thisMonthData.Add(amount).String()
				_, err5 := energyAggregationDb.UserEggEnergyAggregationUpdate(energyAggregation.Id, energyAggregation, "today_data", "this_week_data", "this_month_data")
				if err5 != nil {
					return err5
				}
			}
		}
	}
	return nil
}
